@@ -67,12 +67,12 @@ class COINNDataset(_Dataset):
     def transforms(self, **kw):
         return None
 
-    def path(self, site, inputspec_key, root_dir='baseDirectory'):
-        return self.state[site][root_dir] + _os.sep + self.inputspecs[site][inputspec_key]
+    def path(self, site, root_dir='baseDirectory', inputspec_key='_N/A_'):
+        return _os.path.join(self.state[site][root_dir], self.inputspecs[site].get(inputspec_key, ''))
 
     def add(self, files, cache: dict = None, state: dict = None):
         self.state[state['clientId']] = state
-        self.inputspecs[state['clientId']] = cache['args']
+        self.inputspecs[state['clientId']] = cache
         self._load_indices(site=state['clientId'], files=files, verbose=False)
 
 
