@@ -21,7 +21,7 @@ class COINNLocal:
     _PROMPT_MODE_ = f"Mode must be provided and should be one of {[Mode.TRAIN, Mode.TEST]}."
 
     def __init__(self, cache: dict = None, input: dict = None, state: dict = None,
-                 computation_id=None,
+                 computation_id='nn_computation',
                  mode: str = None,
                  batch_size: int = 16,
                  local_iterations: int = 1,
@@ -62,6 +62,7 @@ class COINNLocal:
         self._args['patience'] = patience if patience else epochs
         self._args['num_folds'] = num_folds
         self._args['split_ratio'] = split_ratio
+
         self._args.update(**kw)
         self._args = _FrozenDict(self._args)
         self._pretrain_args = pretrain_args if pretrain_args else {}
@@ -79,8 +80,17 @@ class COINNLocal:
         for k, sp in self.cache['splits'].items():
             sp = _json.loads(open(self.cache['split_dir'] + _os.sep + sp).read())
             out['data_size'][k] = dict((key, len(sp[key])) for key in sp)
-        for k in SHARED_CACHE:
-            out[k] = self.cache.get(k)
+
+        self.cache['verbose'] = False
+
+        """Keep a copy of default args"""
+        frozen_args = {}.fromkeys(self._args)
+        for k in frozen_args:
+            frozen_args[k] = self.cache[k]
+
+        """Share default args to remote and freeze"""
+        out['shared_args'] = frozen_args
+        self.cache['frozen_args'] = _FrozenDict(frozen_args)
         return out
 
     def _attach_global(self, learner):
@@ -97,12 +107,16 @@ class COINNLocal:
         self._attach_global(learner)
         return out
 
-    def _pretrain_local(self, trainer_cls, train_dataset, validation_dataset):
+    def _pretrain_local(self, trainer_cls, datahandle_cls, train_dataset, validation_dataset):
         out = {'phase': Phase.COMPUTATION}
         if self._pretrain_args.get('epochs') and self.cache['pretrain']:
             cache = {**self.cache}
             cache.update(**self._pretrain_args)
-            trainer = trainer_cls(cache=cache, input=self.input, state=self.state)
+            cache.update(cache.get('pretrain_args', {}))
+            trainer = trainer_cls(data_handle=datahandle_cls(
+                cache=self.cache, input=self.input, state=self.state,
+                dataloader_args=self._dataloader_args
+            ))
             trainer.init_nn()
 
             trainer.init_training_cache()
@@ -130,14 +144,12 @@ class COINNLocal:
 
         self.out['phase'] = self.input.get('phase', Phase.INIT_RUNS)
         if self.out['phase'] == Phase.INIT_RUNS:
-            """ Generate folds as specified.   """
+            """ Prepare: setup parameters, generate data folds..."""
             self.cache.update(**self.input)
             for k in self._args:
                 if self.cache.get(k) is None:
                     self.cache[k] = self._args[k]
             self.out.update(**self._init_runs(learner))
-            self.cache['args'] = _FrozenDict({**self.cache})
-            self.cache['verbose'] = False
             self._check_args()
 
         elif self.out['phase'] == Phase.NEXT_RUN:
@@ -154,6 +166,7 @@ class COINNLocal:
             self.out.update(
                 **self._pretrain_local(
                     trainer_cls,
+                    datahandle_cls,
                     learner.trainer.data_handle.get_train_dataset(dataset_cls),
                     learner.trainer.data_handle.get_validation_dataset(dataset_cls))
             )
@@ -198,7 +211,7 @@ class COINNLocal:
                 Once all sites are in 'val_waiting' status, remote issues 'validation' signal.
                 Once all sites run validation phase, they go to 'train_waiting' status.
                 Once all sites are in this status, remote issues 'train' signal
-                 and all sites reshuffle the indices and r esume training.
+                 and all sites reshuffle the indices and resume training.
                 We send the confusion matrix to the remote to accumulate global score for model selection.
                 """
                 self.out.update(**learner.trainer.validation_distributed(dataset_cls))
@@ -206,7 +219,7 @@ class COINNLocal:
 
             if all(m == Mode.TEST for m in learner.global_modes.values()):
                 self.out.update(**learner.trainer.test_distributed(dataset_cls))
-                self.out['mode'] = self.cache['args']['mode']
+                self.out['mode'] = self.cache['frozen_args']['mode']
                 self.out['phase'] = Phase.NEXT_RUN_WAITING
 
         elif self.out['phase'] == Phase.SUCCESS:
